SystemOrganization addCategory: #'Ring-Core-Containers'!

RGNamedDefinition subclass: #RGAbstractContainer
	instanceVariableNames: 'elements'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ring-Core-Containers'!

!RGAbstractContainer commentStamp: 'VeronicaUquillas 5/12/2011 10:59' prior: 0!
This is the abstract class for container-based elements.
Elements are separated in groups by kind (e.g classes, methods, pools, etc.)
Subclasses have to define the kind of collection for a particular group of entities.
For a container the use of a dictionary or another collection (e.g. SortedCollection) to store group of elements is independent.

!

!RGAbstractContainer methodsFor: 'adding/removing' stamp: 'VeronicaUquillas 4/14/2011 12:17'!
addElement: anObject

	anObject addInContainer: self! !

!RGAbstractContainer methodsFor: 'adding/removing' stamp: 'VeronicaUquillas 4/14/2011 20:08'!
addElement: anObject in: aCollection

	aCollection isDictionary
		ifTrue: [ aCollection at: anObject fullName put: anObject ]
		ifFalse:[ aCollection add: anObject ]! !

!RGAbstractContainer methodsFor: 'lookup elements' stamp: 'VeronicaUquillas 4/13/2011 16:07'!
elementNamed: elementName
	| aSymbol found |
	
	aSymbol:= elementName asSymbol.
	elements do:[ :collection| 		
		(found:= self elementNamed: aSymbol in: collection) notNil ifTrue:[ ^found ] ].
	^nil! !

!RGAbstractContainer methodsFor: 'lookup elements' stamp: 'VeronicaUquillas 4/14/2011 14:41'!
elementNamed: elementName in: aCollection
	| aSymbol |
	
	aSymbol:= elementName asSymbol.
	^aCollection isDictionary
		ifTrue: [ aCollection at: aSymbol ifAbsent: [ nil ] ]
		ifFalse:[ aCollection detect:[ :each| each fullName = aSymbol ] ifNone:[ nil ] ]! !

!RGAbstractContainer methodsFor: 'accessing' stamp: 'VeronicaUquillas 4/14/2011 10:21'!
elements
	"Retrieves the elements"
	
	^elements! !

!RGAbstractContainer methodsFor: 'managing elements groups' stamp: 'VeronicaUquillas 4/15/2011 14:34'!
elementsCategorized: aSymbol
	"Retrieves a group of elements.
	If does not exist set an Set"
	
	^elements at: aSymbol ifAbsentPut:[ Set ]! !

!RGAbstractContainer methodsFor: 'managing elements groups' stamp: 'VeronicaUquillas 4/15/2011 09:47'!
elementsCategorized: aSymbol with: aCollection
	"Allows to define other groups of elements with a particular kind of collection"
	
	elements at: aSymbol put: aCollection! !

!RGAbstractContainer methodsFor: 'iterating elements' stamp: 'VeronicaUquillas 4/13/2011 15:21'!
elementsDo: aBlock

	elements values
		do:[ :collection| collection do:[ :each| aBlock value: each ] ]! !

!RGAbstractContainer methodsFor: 'initialize-release' stamp: 'VeronicaUquillas 4/15/2011 09:43'!
flushElements

	elements:= IdentityDictionary new! !

!RGAbstractContainer methodsFor: 'testing' stamp: 'VeronicaUquillas 7/29/2011 10:51'!
includesElement: anRGDefinition

	^anRGDefinition isIncludedInContainer: self! !

!RGAbstractContainer methodsFor: 'testing' stamp: 'VeronicaUquillas 4/13/2011 16:01'!
includesElementNamed: elementName
	| aSymbol |
	
	aSymbol:= elementName asSymbol.
	elements do:[ :collection| 		
		(self includesElementNamed: aSymbol in: collection) ifTrue:[ ^true ] ].
	^false! !

!RGAbstractContainer methodsFor: 'testing' stamp: 'VeronicaUquillas 4/13/2011 16:03'!
includesElementNamed: elementName in: aCollection
	| aSymbol |
	
	aSymbol:= elementName asSymbol.
	^aCollection isDictionary
		ifTrue: [ aCollection includesKey:  aSymbol ]
		ifFalse:[ (aCollection detect:[ :each| each name = aSymbol ] ifNone:[ nil ]) notNil ]! !

!RGAbstractContainer methodsFor: 'initialize-release' stamp: 'VeronicaUquillas 4/14/2011 10:20'!
initialize

	super initialize.
	elements:= IdentityDictionary new.! !

!RGAbstractContainer methodsFor: 'adding/removing' stamp: 'VeronicaUquillas 4/14/2011 12:17'!
removeElement: anObject

	anObject removeFromContainer: self! !

!RGAbstractContainer methodsFor: 'adding/removing' stamp: 'VeronicaUquillas 4/14/2011 12:25'!
removeElement: anObject from: aCollection

	aCollection isDictionary
		ifTrue: [ aCollection removeKey: anObject fullName ifAbsent:[ ] ]
		ifFalse:[ aCollection remove: anObject ifAbsent:[ ] ]! !

!RGAbstractContainer methodsFor: 'managing elements groups' stamp: 'VeronicaUquillas 4/15/2011 09:48'!
removeElementsCategorized: aSymbol
	"Deletes a group of elements"
	
	elements removeKey: aSymbol ifAbsent:[ ]! !

RGAbstractContainer subclass: #RGContainer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ring-Core-Containers'!

!RGContainer commentStamp: 'VeronicaUquillas 4/19/2011 15:48' prior: 0!
A concrete container already knows how to treat classes and methods. !

!RGContainer methodsFor: 'adding/removing' stamp: 'VeronicaUquillas 5/17/2011 12:53'!
addClass: aRGBehaviorDefinition
	"aRGBehaviorDefinition has to be a class, trait or metaclass"

	(aRGBehaviorDefinition isClass
		or:[ aRGBehaviorDefinition isTrait ]) ifFalse:[ ^self ].
	self addElement: aRGBehaviorDefinition in: self classes! !

!RGContainer methodsFor: 'adding/removing' stamp: 'VeronicaUquillas 4/28/2011 15:50'!
addClassNamed: className 
	"Creates a class with the given name"
	| cls |
	
	cls:= self factory createClassNamed: className.
	self addClass: cls! !

!RGContainer methodsFor: 'adding/removing' stamp: 'VeronicaUquillas 4/28/2011 15:50'!
addMethod: aRGMethodDefinition
	"aRGMethodDefinition needs to satisfy the status isMethod"

	aRGMethodDefinition isMethod ifFalse:[ ^self ].
	self addElement: aRGMethodDefinition in: self methods! !

!RGContainer methodsFor: 'adding/removing' stamp: 'VeronicaUquillas 7/27/2011 15:18'!
addPackage: aRGPackage
	"adds a child package"

	aRGPackage isPackage ifFalse:[ ^self ].
	self addElement: aRGPackage in: self packages! !

!RGContainer methodsFor: 'adding/removing' stamp: 'VeronicaUquillas 7/27/2011 15:18'!
addPackageNamed: packageName
	"adds a child package with the given name"
	| cPackage |

	(self includesPackageNamed: packageName) ifTrue:[ ^self ].
	cPackage:= self factory createPackageNamed: packageName.
	self addPackage: cPackage! !

!RGContainer methodsFor: 'adding/removing' stamp: 'VeronicaUquillas 4/28/2011 15:50'!
addTrait: aRGTraitDefinition
	"convenient method"

	self addClass: aRGTraitDefinition! !

!RGContainer methodsFor: 'adding/removing' stamp: 'VeronicaUquillas 4/28/2011 15:51'!
addTraitNamed: traitName 
	"Creates a trait with the given name"
	| trait |
	
	trait:= self factory createTraitNamed: traitName.
	self addClass: trait! !

!RGContainer methodsFor: 'accessing' stamp: 'VeronicaUquillas 4/19/2011 14:10'!
allClasses
	"convenient method"

	^self classes! !

!RGContainer methodsFor: 'iterating' stamp: 'VeronicaUquillas 7/29/2011 10:48'!
allClassesDo: aBlock
	"Evaluate the argument, aBlock, for each class and its metaclass"

	self allClasses do: [ :each|
		aBlock value: each.
		each hasMetaclass
			ifTrue:[ aBlock value: each theMetaClass ] ]! !

!RGContainer methodsFor: 'accessing' stamp: 'VeronicaUquillas 8/8/2011 10:21'!
allMethods
	"convenient method"

	^self methods! !

!RGContainer methodsFor: 'accessing' stamp: 'VeronicaUquillas 4/19/2011 14:12'!
allTraits
	"convenient method"
	
	^self traits! !

!RGContainer methodsFor: 'iterating' stamp: 'VeronicaUquillas 7/29/2011 10:48'!
allTraitsDo: aBlock
	"Evaluate the argument, aBlock, for each trait"

	self allTraits do: [ :each|
		aBlock value: each.
		each hasMetaclass
			ifTrue:[ aBlock value: each theMetaClass ] ]! !

!RGContainer methodsFor: 'lookup by name' stamp: 'VeronicaUquillas 7/28/2011 16:59'!
classNamed: className
	"Retrieves an RGBehaviorDefinition object.
	className could be theMetaClass name"
	
	^self classOrTraitNamed: className! !

!RGContainer methodsFor: 'accessing' stamp: 'VeronicaUquillas 4/19/2011 14:12'!
classNames
	"Retrieves all class names - including traits"
	
	^self allClasses collect:[ :class|  class name ]! !

!RGContainer methodsFor: 'lookup by name' stamp: 'VeronicaUquillas 7/28/2011 16:48'!
classOrTraitNamed: className
	"A class or metaclass can be reached by its name"
	| theClass |

	className ifNil: [^nil ].
	theClass:= self elementNamed: (self nonMetaClassNameOf: className) in: self classes.
	theClass ifNil:[ ^nil ].
	
	^(self isMetaclassName: className)
		ifTrue: [ theClass theMetaClass ]
		ifFalse:[ theClass ] ! !

!RGContainer methodsFor: 'accessing' stamp: 'VeronicaUquillas 4/19/2011 14:11'!
classes
	"Retrieves classes (traits are included)"

	^self elements at: #classes ifAbsent:[ #() ]! !

!RGContainer methodsFor: 'accessing' stamp: 'VeronicaUquillas 7/27/2011 15:20'!
classes: aCollection
	"Set the classes collection"

	self elements at: #classes put: aCollection! !

!RGContainer methodsFor: 'accessing' stamp: 'VeronicaUquillas 7/27/2011 15:48'!
extensionMethods

	^self methods values select:[ :mth| mth isExtension ]! !

!RGContainer methodsFor: 'testing' stamp: 'VeronicaUquillas 4/19/2011 14:11'!
hasClasses

	^self classes notEmpty! !

!RGContainer methodsFor: 'testing' stamp: 'VeronicaUquillas 4/19/2011 14:13'!
hasMethods

	^self methods notEmpty! !

!RGContainer methodsFor: 'testing' stamp: 'VeronicaUquillas 4/28/2011 15:48'!
includesClass: aRGAbstractClassDefinition

	^self classes includes: aRGAbstractClassDefinition! !

!RGContainer methodsFor: 'testing' stamp: 'VeronicaUquillas 4/19/2011 14:11'!
includesClassNamed: className
	
	^self includesElementNamed: className in: self classes! !

!RGContainer methodsFor: 'testing' stamp: 'VeronicaUquillas 4/28/2011 15:49'!
includesMethod: aRGMethodDefinition

	^self methods includes: aRGMethodDefinition! !

!RGContainer methodsFor: 'testing' stamp: 'VeronicaUquillas 4/19/2011 14:13'!
includesMethodNamed: fullSelectorName
	
	^self includesElementNamed: fullSelectorName in: self methods! !

!RGContainer methodsFor: 'testing' stamp: 'VeronicaUquillas 7/27/2011 15:18'!
includesPackage: aRGPackage
	"Verifies if a child package == aRGPackage exists"
	
	^self packages includes: aRGPackage! !

!RGContainer methodsFor: 'testing' stamp: 'VeronicaUquillas 7/27/2011 15:18'!
includesPackageNamed: packageName
	"Verifies if a child package with name = packageName exists"
	
	^self includesElementNamed: packageName in: self packages! !

!RGContainer methodsFor: 'initialize-release' stamp: 'VeronicaUquillas 7/27/2011 15:19'!
initialize

	super initialize.
	self classes: IdentityDictionary new.
	self methods: IdentityDictionary new.
	self packages: IdentityDictionary new.! !

!RGContainer methodsFor: 'utilities' stamp: 'VeronicaUquillas 4/19/2011 14:11'!
isMetaclassName: aSymbol
	"Validates if the arguments corresponds to a nonMetaClass"
	
	^(aSymbol
		indexOfSubCollection: ' class'
		startingAt: 1) > 0! !

!RGContainer methodsFor: 'lookup by name' stamp: 'VeronicaUquillas 4/19/2011 14:11'!
metaclassNamed: metaclassName
	| theClass |
	
	theClass:= (self classOrTraitNamed: metaclassName).
	theClass notNil
	ifTrue:[ ^theClass isMeta 
		      ifTrue: [ theClass ]
		      ifFalse:[ theClass theMetaClass ] ].
	
	^nil! !

!RGContainer methodsFor: 'lookup by name' stamp: 'VeronicaUquillas 4/19/2011 14:13'!
methodNamed: fullSelectorName

	^self elementNamed: fullSelectorName in: self methods! !

!RGContainer methodsFor: 'accessing' stamp: 'VeronicaUquillas 4/19/2011 14:13'!
methods
	"Retrieves methods"
	
	^self elements at: #methods ifAbsent:[ #() ]! !

!RGContainer methodsFor: 'accessing' stamp: 'VeronicaUquillas 7/27/2011 15:20'!
methods: aCollection
	"Set the methods collection"
	
	self elements at: #methods put: aCollection! !

!RGContainer methodsFor: 'iterating' stamp: 'VeronicaUquillas 4/19/2011 14:13'!
methodsDo: aBlock

	self methods
		do:[ :each| aBlock value: each ]! !

!RGContainer methodsFor: 'utilities' stamp: 'VeronicaUquillas 4/19/2011 14:11'!
nonMetaClassNameOf: aSymbol
	"Rejects the prefix ' class' or ' classTrait' of the argument"
	| index |
	
	index := aSymbol
				indexOfSubCollection: ' class'
				startingAt: 1
				ifAbsent: [ ^aSymbol asSymbol ].

	^(aSymbol 
		copyFrom: 1
		to: index - 1) asSymbol! !

!RGContainer methodsFor: 'lookup by name' stamp: 'VeronicaUquillas 7/27/2011 15:18'!
packageNamed: packageName
	"Look for a child package named packageName"
	
	^self elementNamed: packageName in: self packages! !

!RGContainer methodsFor: 'accessing' stamp: 'VeronicaUquillas 7/27/2011 15:18'!
packages
	"Retrieves the children packages defined in the receiver"

	^self elementsCategorized: #packages! !

!RGContainer methodsFor: 'accessing' stamp: 'VeronicaUquillas 7/27/2011 15:18'!
packages: aCollection

	self elementsCategorized: #packages with: aCollection! !

!RGContainer methodsFor: 'adding/removing' stamp: 'VeronicaUquillas 4/28/2011 15:52'!
removeClass: aRGAbstractClassDefinition

	self removeElement: aRGAbstractClassDefinition from: self classes! !

!RGContainer methodsFor: 'adding/removing' stamp: 'VeronicaUquillas 4/28/2011 15:52'!
removeMethod: aRGMethodDefinition

	self removeElement: aRGMethodDefinition from: self methods! !

!RGContainer methodsFor: 'adding/removing' stamp: 'VeronicaUquillas 7/27/2011 15:18'!
removePackage: aRGPackage
	"removes a child aRGPackage"

	aRGPackage isPackage ifFalse:[ ^self ].
	self removeElement: aRGPackage from: self packages! !

!RGContainer methodsFor: 'adding/removing' stamp: 'VeronicaUquillas 4/28/2011 15:52'!
removeTrait: aRGTraitDefinition
	"convenient method"

	self removeClass: aRGTraitDefinition! !

!RGContainer methodsFor: 'lookup by name' stamp: 'VeronicaUquillas 7/28/2011 16:30'!
traitNamed: traitName
	"Retrieves an RGTraitDefinition object.
	traitName could be theMetaClass name"
	| trait |
	
	^(trait:= self classOrTraitNamed: traitName) isTrait
	ifTrue:[ trait ]
	ifFalse:[ nil ]! !

!RGContainer methodsFor: 'accessing' stamp: 'VeronicaUquillas 4/19/2011 14:12'!
traitNames
	"Retrieves all the names of traits"

	^self allTraits collect:[ :trait|  trait name ]! !

!RGContainer methodsFor: 'accessing' stamp: 'VeronicaUquillas 4/19/2011 14:12'!
traits

	^self classes select:[ :each| each isTrait ]! !

!RGContainer methodsFor: 'adding/removing' stamp: 'VeronicaUquillas 4/19/2011 14:24'!
withoutClasses

	self removeElementsCategorized: #classes! !

!RGContainer methodsFor: 'adding/removing' stamp: 'VeronicaUquillas 4/19/2011 14:23'!
withoutMethods
	
	self removeElementsCategorized: #methods! !

RGContainer subclass: #RGNamespace
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ring-Core-Containers'!

!RGNamespace commentStamp: 'VeronicaUquillas 4/20/2011 16:34' prior: 0!
A namespace is a container that in addition to managing classes and methods, it also knows how to treat pools and global variables!

!RGNamespace methodsFor: 'adding elements' stamp: 'VeronicaUquillas 4/14/2011 20:09'!
addElement: anObject in: aCollection

	super addElement: anObject in: aCollection.
	anObject environment: self! !

!RGNamespace methodsFor: 'adding elements' stamp: 'VeronicaUquillas 4/14/2011 14:30'!
addGlobalVariable: anORGlobalVariableDefinition
	"adds anORGlobalVariableDefinition to the globals dictionary if satisfies it #isGlobalVariable"

	anORGlobalVariableDefinition isGlobalVariable ifFalse:[ ^self ].
	self addElement: anORGlobalVariableDefinition in: self globalVariables! !

!RGNamespace methodsFor: 'adding elements' stamp: 'VeronicaUquillas 4/28/2011 16:23'!
addGlobalVariableNamed: globalVariableName
	"creates a RGGlobalVariableDefinition with the given name.
	It's verified that a pool with that name does not exist"
	| gv |

	(self includesGlobalVariableNamed: globalVariableName) ifTrue: [ ^self ].
	gv:= self factory createGlobalVariableNamed: globalVariableName.
	self addGlobalVariable: gv.! !

!RGNamespace methodsFor: 'adding elements' stamp: 'VeronicaUquillas 4/14/2011 14:29'!
addPool: anORPoolDefinition
	"adds anORPoolDefinition to the pools dictionary if satisfies it #isPool"

	anORPoolDefinition isPool ifFalse:[ ^self ].
	self addElement: anORPoolDefinition in: self pools! !

!RGNamespace methodsFor: 'adding elements' stamp: 'VeronicaUquillas 4/28/2011 16:24'!
addPoolNamed: poolName
	"creates a RGPoolDefinition with the given name.
	It's verified that a pool with that name does not exist"
	| pool |

	(self includesPoolNamed: poolName) ifTrue: [ ^self ].
	pool:= self factory createSharedPoolNamed: poolName.
	self addPool: pool.! !

!RGNamespace methodsFor: 'accessing global variables' stamp: 'VeronicaUquillas 4/14/2011 15:39'!
globalVariableNamed: globalVariableName

	^self elementNamed: globalVariableName in: self globalVariables! !

!RGNamespace methodsFor: 'accessing global variables' stamp: 'VeronicaUquillas 4/15/2011 09:49'!
globalVariables
	"retrieves global variables"

	^self elementsCategorized: #globalVariables! !

!RGNamespace methodsFor: 'accessing global variables' stamp: 'VeronicaUquillas 4/15/2011 09:47'!
globalVariables: aCollection

	self elementsCategorized: #globalVariables with: aCollection! !

!RGNamespace methodsFor: 'testing' stamp: 'VeronicaUquillas 4/14/2011 15:36'!
includesGlobalVariable: anORGlobalVariableDefinition

	^self globalVariables includes: anORGlobalVariableDefinition! !

!RGNamespace methodsFor: 'testing' stamp: 'VeronicaUquillas 4/14/2011 15:01'!
includesGlobalVariableNamed: globalVariableName
	
	^self includesElementNamed: globalVariableName in: self globalVariables! !

!RGNamespace methodsFor: 'testing' stamp: 'VeronicaUquillas 4/14/2011 15:35'!
includesPool: anORPoolDefinition

	^self pools includes: anORPoolDefinition! !

!RGNamespace methodsFor: 'testing' stamp: 'VeronicaUquillas 4/14/2011 15:01'!
includesPoolNamed: poolName
	
	^self includesElementNamed: poolName in: self pools! !

!RGNamespace methodsFor: 'initialize-release' stamp: 'VeronicaUquillas 4/20/2011 16:40'!
initialize

	super initialize.
	self globalVariables: IdentityDictionary new.
	self pools: IdentityDictionary new.! !

!RGNamespace methodsFor: 'testing' stamp: 'VeronicaUquillas 4/19/2011 15:31'!
isNamespace

	^true! !

!RGNamespace methodsFor: 'accessing pools' stamp: 'VeronicaUquillas 4/14/2011 14:40'!
poolNamed: poolName

	^self elementNamed: poolName in: self pools! !

!RGNamespace methodsFor: 'accessing pools' stamp: 'VeronicaUquillas 4/15/2011 09:49'!
pools
	"retrieves the shares pools"

	^self elementsCategorized: #pools! !

!RGNamespace methodsFor: 'accessing pools' stamp: 'VeronicaUquillas 4/15/2011 09:47'!
pools: aCollection

	self elementsCategorized: #pools with: aCollection! !

!RGNamespace methodsFor: 'removing elements' stamp: 'VeronicaUquillas 4/14/2011 15:13'!
removeGlobalVariable: anORGlobalVariableDefinition
	"removes anORGlobalVariableDefinition from the globals dictionary if satisfies it #isGlobalVariable"

	anORGlobalVariableDefinition isGlobalVariable ifFalse:[ ^self ].
	self removeElement: anORGlobalVariableDefinition from: self globalVariables! !

!RGNamespace methodsFor: 'removing elements' stamp: 'VeronicaUquillas 4/14/2011 15:13'!
removePool: anORPoolDefinition
	"removes anORPoolDefinition from the pools dictionary if satisfies it #isPool"

	anORPoolDefinition isPool ifFalse:[ ^self ].
	self removeElement: anORPoolDefinition from: self pools! !

!RGNamespace methodsFor: 'removing elements' stamp: 'VeronicaUquillas 4/20/2011 16:35'!
withoutGlobalVariables

	self removeElementsCategorized: #globalVariables! !

!RGNamespace methodsFor: 'removing elements' stamp: 'VeronicaUquillas 4/20/2011 16:35'!
withoutPools

	self removeElementsCategorized: #pools! !

RGContainer subclass: #RGPackage
	instanceVariableNames: 'package'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ring-Core-Containers'!

!RGPackage commentStamp: 'VeronicaUquillas 4/20/2011 17:18' prior: 0!
RGPackage is the representation for packages.
A package manages classes, methods and children packages as elements.
A package can also know in which package is defined
!

!RGPackage methodsFor: 'adding/removing' stamp: 'VeronicaUquillas 7/28/2011 10:58'!
addElement: aRGDefinition in: aCollection

	super addElement: aRGDefinition in: aCollection.
	aRGDefinition package: self! !

!RGPackage methodsFor: 'adding/removing' stamp: 'VeronicaUquillas 8/8/2011 10:20'!
addExtensionMethod: aRGMethodDefinition
	"aRGMethodDefinition needs to satisfy the status isMethod"

	aRGMethodDefinition isMethod ifFalse:[ ^self ].
	self addElement: aRGMethodDefinition in: self extensionMethods! !

!RGPackage methodsFor: 'managing container' stamp: 'VeronicaUquillas 7/29/2011 10:52'!
addInContainer: anRGContainer

	anRGContainer addPackage: self! !

!RGPackage methodsFor: 'accessing' stamp: 'VeronicaUquillas 8/8/2011 10:28'!
allClasses
	"Retrieves all the classes including the ones holding the extension methods"
	
	^self classes, self classesOfExtensionMethods ! !

!RGPackage methodsFor: 'accessing' stamp: 'VeronicaUquillas 8/8/2011 10:22'!
allMethods
	"Methods in a package are separated in two dictionaries"
	
	^self methods, self extensionMethods ! !

!RGPackage methodsFor: 'accessing' stamp: 'VeronicaUquillas 8/8/2011 10:30'!
allTraits
	"Retrieves all the traits including the ones holding the extension methods"
	
	^self allClasses select:[ :each| each isTrait ]! !

!RGPackage methodsFor: 'accessing' stamp: 'VeronicaUquillas 8/8/2011 10:28'!
classesOfExtensionMethods
	"Retrieves the classes holding the extension methods and returns a dictionary"
	|  classesOfExt |
	classesOfExt := Dictionary new.
	
	self extensionMethods
		do:[ :mth| (classesOfExt includesKey: mth parentName)
					ifFalse:[ classesOfExt at: mth parentName put: mth parent ] ].
	
	^classesOfExt! !

!RGPackage methodsFor: 'accessing' stamp: 'VeronicaUquillas 8/8/2011 10:20'!
extensionMethods
	"Retrieves extension methods"
	
	^self elements at: #extensionMethods ifAbsent:[ #() ]! !

!RGPackage methodsFor: 'accessing' stamp: 'VeronicaUquillas 8/8/2011 10:19'!
extensionMethods: aCollection
	"Set the extension methods collection"
	
	self elements at: #extensionMethods put: aCollection! !

!RGPackage methodsFor: 'initialize-release' stamp: 'VeronicaUquillas 8/8/2011 10:20'!
initialize

	super initialize.
	self extensionMethods: IdentityDictionary new.! !

!RGPackage methodsFor: 'testing' stamp: 'VeronicaUquillas 3/20/2011 16:52'!
isPackage

	^true! !

!RGPackage methodsFor: 'lookup' stamp: 'VeronicaUquillas 4/28/2011 14:35'!
lookupClassNamed: className
	"Look for the class named className in the receiver's classes or in any of its package children"
	| class |
	
	class:= self classNamed: className.
	class ifNotNil:[ ^class ].
	
	self packages
		do:[ :each| class:= each lookupClassNamed: className.
				    class ifNotNil:[ ^class ] ].
	^nil! !

!RGPackage methodsFor: 'accessing' stamp: 'VeronicaUquillas 4/11/2011 12:08'!
package
	"Retrieves the  package in which self is contained"

	^package! !

!RGPackage methodsFor: 'accessing' stamp: 'VeronicaUquillas 4/20/2011 17:15'!
package: aRGPackage
	"Sets the outer package of the receiver if exists"

	package:= aRGPackage! !

!RGPackage methodsFor: 'accessing' stamp: 'VeronicaUquillas 4/15/2011 14:41'!
parent
	"Retrieves the environment linked to this package"
	
	^self environment! !

!RGPackage methodsFor: 'adding/removing' stamp: 'VeronicaUquillas 8/8/2011 10:21'!
removeExtensionMethod: aRGMethodDefinition

	self removeElement: aRGMethodDefinition from: self extensionMethods! !

RGContainer subclass: #RGSlice
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ring-Core-Containers'!

!RGSlice commentStamp: 'VeronicaUquillas 4/22/2011 13:36' prior: 0!
A slice is an arbitrary container of elements (working unit)
By inheritance it knows how to treat classes and methods
!

!RGSlice methodsFor: 'adding/removing' stamp: 'VeronicaUquillas 4/22/2011 13:50'!
addElement: aRGDefinition in: aCollection

	super addElement: aRGDefinition in: aCollection.
	aRGDefinition environment: self! !

!RGSlice methodsFor: 'testing' stamp: 'VeronicaUquillas 4/19/2011 15:30'!
isSlice

	^true! !

RGAbstractContainer subclass: #RGOrganization
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ring-Core-Containers'!

!RGOrganization methodsFor: 'accessing' stamp: 'VeronicaUquillas 5/12/2011 11:01'!
categories
	"Retrieves categories in the system (tag-packages)"

	^self elements at: #categories ifAbsent:[ #() ]! !

!RGOrganization methodsFor: 'accessing' stamp: 'VeronicaUquillas 5/12/2011 13:45'!
categories: aCollection
	"Set the categories collection"

	self elements at: #categories put: aCollection! !

!RGOrganization methodsFor: 'accessing' stamp: 'VeronicaUquillas 5/13/2011 00:21'!
fullName

	^#organization! !

!RGOrganization methodsFor: 'accessing' stamp: 'VeronicaUquillas 5/12/2011 11:01'!
initialize

	super initialize.
	self categories: OrderedCollection new.! !

!RGOrganization methodsFor: 'testing' stamp: 'VeronicaUquillas 5/12/2011 13:41'!
isOrganization

	^true! !

!RGOrganization methodsFor: 'testing' stamp: 'VeronicaUquillas 7/18/2011 14:32'!
isSameRevisionAs: aRGOrganization
	"This method look for equality of the properties of the receiver"

	^(self class = aRGOrganization class)
		and:[ self categories = aRGOrganization categories ]! !

!RGFactory methodsFor: '*Ring-Core-Containers' stamp: 'VeronicaUquillas 4/28/2011 17:23'!
createNamespaceNamed: aString

	^self createInstanceOfClass: self namespaceClass named: aString! !

!RGFactory methodsFor: '*Ring-Core-Containers' stamp: 'VeronicaUquillas 5/12/2011 11:02'!
createOrganization

	^self organizationClass new! !

!RGFactory methodsFor: '*Ring-Core-Containers' stamp: 'VeronicaUquillas 4/28/2011 15:20'!
createPackageNamed: aString

	^self createInstanceOfClass: self packageClass named: aString! !

!RGFactory methodsFor: '*Ring-Core-Containers' stamp: 'VeronicaUquillas 4/28/2011 17:23'!
createSliceNamed: aString

	^self createInstanceOfClass: self sliceClass named: aString! !

!RGFactory methodsFor: '*Ring-Core-Containers' stamp: 'VeronicaUquillas 4/28/2011 17:23'!
namespaceClass

	^RGNamespace! !

!RGFactory methodsFor: '*Ring-Core-Containers' stamp: 'VeronicaUquillas 5/12/2011 11:02'!
organizationClass

	^RGOrganization! !

!RGFactory methodsFor: '*Ring-Core-Containers' stamp: 'VeronicaUquillas 4/27/2011 22:07'!
packageClass

	^RGPackage! !

!RGFactory methodsFor: '*Ring-Core-Containers' stamp: 'VeronicaUquillas 4/27/2011 22:07'!
sliceClass

	^RGSlice! !
